/*
 * this file is part of the "minidock" project
 * Copyright (C) 2003,2006 by Jean Philippe GUILLEMIN <jp.guillemin@free.fr>
 * license: This software is under GPL license
 * rev: 1.9  
 */


#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <getopt.h>
#include <sysexits.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/resource.h>
#include <sys/wait.h>
#include <gtk/gtk.h>
#include <glib.h>
#include "minidock.h"
#include <X11/Xutil.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>


// Start main *******************************************************************

GtkWidget *dockapp;
static struct minidock *minidock1, *minidock2, *minidock3, *minidock4;


int main( int argc, char *argv[] ) {

// Here we parse command line args and configfile *******************************************************************

	char defaultcf[128];
	char *configfile = NULL;
	char homecf[128];
	char *homedir = NULL;
	
	strncpy(defaultcf, __CONFPATH, sizeof(defaultcf));	
	strcat(defaultcf, "/");	
	strcat(defaultcf, __CONFFILE);	
	
	homedir = getenv("HOME");
	
	strncpy(homecf, homedir, sizeof(homecf));
	strcat(homecf, "/");	
	strcat(homecf, ".minidock.cf");

	gtk_set_locale ();

	int test1, test2;
	if ((test1 = access(homecf, F_OK)) == -1){
		if ((test2 = fcopy(defaultcf, homecf)) == EXIT_FAILURE){
			fprintf(stderr,"Error creating config file %s !\n",homecf);
		}
	}
	
	int i;
	
	if (argc < 2){
		configfile = homecf;
	}else{
		while ((i = getopt(argc, argv, "hc:")) != EOF){
			switch (i){
				case 'c': /* config file */
					configfile = optarg;
					break;
				case 'h': usage(homecf, defaultcf); exit (EXIT_SUCCESS);
			}
		}
	}
	
	
	minidock1 = malloc(sizeof(struct minidock));
	minidock2 = malloc(sizeof(struct minidock));
	minidock3 = malloc(sizeof(struct minidock));
	minidock4 = malloc(sizeof(struct minidock));
	memset(minidock1, 0, sizeof(struct minidock));
	memset(minidock2, 0, sizeof(struct minidock));
	memset(minidock3, 0, sizeof(struct minidock));
	memset(minidock4, 0, sizeof(struct minidock));

	strncpy (minidock1->param_icon, "icon1", 128);
	strncpy (minidock1->param_cmd, "command1", 128);
	strncpy (minidock2->param_icon, "icon2", 128);
	strncpy (minidock2->param_cmd, "command2", 128);
	strncpy (minidock3->param_icon, "icon3", 128);
	strncpy (minidock3->param_cmd, "command3", 128);
	strncpy (minidock4->param_icon, "icon4", 128);
	strncpy (minidock4->param_cmd, "command4", 128);
	strncpy (minidock1->configfile, configfile, 128);
	strncpy (minidock2->configfile, configfile, 128);
	strncpy (minidock3->configfile, configfile, 128);
	strncpy (minidock4->configfile, configfile, 128);	
					
	gtk_init(&argc, &argv);
	dockapp = (GtkWidget *) build_dockapp(configfile);
	gtk_main ();
	return(0);
} // end main


	/* Event handler for window closed */
	static gint window_closed(GtkWidget*   w,
		GdkEventAny* e,
		gpointer data)
	{
		/* Terminate the event loop, then tell
		* GTK+ it's OK to delete the window
		*/
		gtk_main_quit();
		return FALSE;
	}

GtkWidget * build_dockapp(char *configfile) {
	GtkWidget *mainwin;
	Window xmw;
	XWMHints *wm_hints;
	GtkWidget *mainbox;
	GtkWidget *table;
	GtkWidget *box1, *box2, *box3, *box4;
	GdkPixbuf *pixbuf1,*pixbuf2,*pixbuf3,*pixbuf4;
	GtkWidget *pixmap_widget1,*pixmap_widget2,*pixmap_widget3,*pixmap_widget4;
	GtkIconTheme *icon_theme;
	icon_theme = gtk_icon_theme_get_default();	

	
	parse_from_config(minidock1->configfile, minidock1->param_icon, minidock1->icon);
	parse_from_config(minidock1->configfile, minidock1->param_cmd, minidock1->command);
	
	parse_from_config(minidock2->configfile, minidock2->param_icon, minidock2->icon);
	parse_from_config(minidock2->configfile, minidock2->param_cmd, minidock2->command);
	
	parse_from_config(minidock3->configfile, minidock3->param_icon, minidock3->icon);
	parse_from_config(minidock3->configfile, minidock3->param_cmd, minidock3->command);
	
	parse_from_config(minidock4->configfile, minidock4->param_icon, minidock4->icon);
	parse_from_config(minidock4->configfile, minidock4->param_cmd, minidock4->command);
	
	// GTK stuff *******************************************************************

	mainwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);

	mainbox = gtk_event_box_new();
	gtk_widget_set_usize(mainbox, 52, 52);
	gtk_container_add (GTK_CONTAINER (mainwin), mainbox);
	
	/* Set up the event handler */
	gtk_signal_connect(GTK_OBJECT(mainwin),
		"delete_event",
		GTK_SIGNAL_FUNC(window_closed),
		NULL);
	
	table = gtk_table_new(2,2, TRUE);
	
	box1 = gtk_event_box_new();
	pixbuf1 = gtk_icon_theme_load_icon (icon_theme, minidock1->icon, 24, 0, NULL);
	pixmap_widget1 = gtk_image_new_from_pixbuf(pixbuf1);
	gtk_container_add(GTK_CONTAINER(box1), pixmap_widget1);

	box2 = gtk_event_box_new();
	pixbuf2 = gtk_icon_theme_load_icon (icon_theme, minidock2->icon, 24, 0, NULL);
	pixmap_widget2 = gtk_image_new_from_pixbuf(pixbuf2);
	gtk_container_add(GTK_CONTAINER(box2), pixmap_widget2);

	box3 = gtk_event_box_new();
	pixbuf3 = gtk_icon_theme_load_icon (icon_theme, minidock3->icon, 24, 0, NULL);
	pixmap_widget3 = gtk_image_new_from_pixbuf(pixbuf3);
	gtk_container_add(GTK_CONTAINER(box3), pixmap_widget3);
	
	box4 = gtk_event_box_new();
	pixbuf4 = gtk_icon_theme_load_icon (icon_theme, minidock4->icon, 24, 0, NULL);
	pixmap_widget4 = gtk_image_new_from_pixbuf(pixbuf4);
	gtk_container_add(GTK_CONTAINER(box4), pixmap_widget4);
	
	
	gtk_container_add (GTK_CONTAINER (mainbox), table);
	
	gtk_table_attach_defaults(GTK_TABLE(table), box1, 0,1, 0,1);
	gtk_table_attach_defaults(GTK_TABLE(table), box2, 1,2, 0,1);
	gtk_table_attach_defaults(GTK_TABLE(table), box3, 0,1, 1,2);
	gtk_table_attach_defaults(GTK_TABLE(table), box4, 1,2, 1,2);

	gtk_signal_connect (GTK_OBJECT(box1), "button_press_event", GTK_SIGNAL_FUNC(launchonclick), minidock1);
	gtk_signal_connect (GTK_OBJECT(box2), "button_press_event", GTK_SIGNAL_FUNC(launchonclick), minidock2);
	gtk_signal_connect (GTK_OBJECT(box3), "button_press_event", GTK_SIGNAL_FUNC(launchonclick), minidock3);
	gtk_signal_connect (GTK_OBJECT(box4), "button_press_event", GTK_SIGNAL_FUNC(launchonclick), minidock4);
	
	gtk_widget_show_all (mainwin);
	
	// Make it a dockapp
	xmw = GDK_WINDOW_XWINDOW(mainwin->window);

	wm_hints = XAllocWMHints();
	wm_hints->window_group = xmw;
	wm_hints->icon_window = GDK_WINDOW_XWINDOW(mainbox->window);
	wm_hints->icon_x = 0;
	wm_hints->icon_y = 0; 
	wm_hints->initial_state = WithdrawnState;
	wm_hints->flags = StateHint |
		IconPositionHint |
		WindowGroupHint |
		IconWindowHint;
	
	XSetWMHints(GDK_DISPLAY(), xmw, wm_hints);
	
	XFree(wm_hints);
	return mainwin;

}




/* the function to execute command when left button is clicked *************************************
	or change the command and pixmap icon when right button is 2-clicked */

void launchonclick
		(GtkWidget *event_box, 
		GdkEventButton *event,
		struct minidock *data)
{
		pid_t pid, w;
		int status;
		/* left button of mouse is clicked */
        if(event->button == 1){
			pid = fork();
			if (pid == 0) {
				if (system (data->command)){
					fprintf(stderr,"Unable to launch command %s", data->command); 
				}
			} else if (pid < 0) {
				perror ("Unable to fork");   
			} else if (pid > 0) {
				w = waitpid(pid, &status, WNOHANG);

			}

			return;
       }
	   
	   /* right button of mouse is 2-clicked */
        if(event->button == 3)
        {
			if(event->type == GDK_2BUTTON_PRESS){
			
				/* create a new text entry window for entering the new command */
				GtkWidget *dialog;
				GtkWidget *button;
				GtkWidget *entry1, *entry2;
				
				// window parameters
				dialog = gtk_dialog_new();
				gtk_window_set_title (GTK_WINDOW (dialog), "command & icon");
				gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER); 
				gtk_window_resize (GTK_WINDOW (dialog), 250, 50);
				
				// entry box parameters
				entry1 = gtk_entry_new ();
				entry2 = gtk_entry_new ();
				gtk_entry_set_has_frame (GTK_ENTRY (entry1), TRUE);
				gtk_entry_set_has_frame (GTK_ENTRY (entry2), TRUE);
				gtk_entry_set_max_length (GTK_ENTRY (entry1), 128);
				gtk_entry_set_max_length (GTK_ENTRY (entry2), 128);
				gtk_entry_set_activates_default (GTK_ENTRY (entry1), TRUE);
				gtk_entry_set_activates_default (GTK_ENTRY (entry2), TRUE);
				
				gtk_signal_connect (GTK_OBJECT (entry1), "changed", GTK_SIGNAL_FUNC (enter_callback1), data);
				gtk_signal_connect (GTK_OBJECT (entry2), "changed", GTK_SIGNAL_FUNC (enter_callback2), data);
				
				gtk_entry_set_text (GTK_ENTRY (entry1), data->command);
				gtk_entry_set_text (GTK_ENTRY (entry2), data->icon);
				gtk_editable_set_editable (GTK_EDITABLE (entry1), TRUE);
				gtk_entry_set_visibility (GTK_ENTRY (entry1), TRUE);
				gtk_editable_set_editable (GTK_EDITABLE (entry2), TRUE);
				gtk_entry_set_visibility (GTK_ENTRY (entry2), TRUE);
				
				// button parameters
				button = gtk_button_new_from_stock("gtk-ok");
				GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
				gtk_signal_connect (GTK_OBJECT (button), "clicked", GTK_SIGNAL_FUNC (close_callback), data);
				
				gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 5);
				gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->action_area), button);
				gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), entry1);
				gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), entry2);
				gtk_window_set_default(GTK_WINDOW (dialog), button);
				gtk_widget_show_all(dialog);

				return;
			}
       	}
} 


/* some callback to handle change of values in entry boxes************************* */
void enter_callback1( 
			GtkWidget *entry,
			struct minidock *data){
  	char entry_text[sizeof(data->command)];
	memset(entry_text, 0, sizeof(entry_text));
  	strncpy (entry_text, gtk_entry_get_text (GTK_ENTRY (entry)), sizeof(entry_text));
	memset(data->command, 0, sizeof(data->command));
	strncpy (data->command, entry_text, sizeof(data->command));
	
}
void enter_callback2( 
			GtkWidget *entry,
			struct minidock *data){
  	char entry_text[sizeof(data->icon)];
	memset(entry_text, 0, sizeof(entry_text));
  	strncpy (entry_text, gtk_entry_get_text (GTK_ENTRY (entry)), sizeof(entry_text));
	memset(data->icon, 0, sizeof(data->icon));
	strncpy (data->icon, entry_text, sizeof(data->icon));	
}

void close_callback( 
			GtkWidget *entry,
			struct minidock *data){
	
	GtkWidget *parent;
  	replace_value_in_config(data->configfile, data->param_cmd, data->command);
	replace_value_in_config(data->configfile, data->param_icon, data->icon);
	parent = gtk_widget_get_ancestor(entry, GTK_TYPE_WINDOW); 
	gtk_widget_destroy(parent);
	gtk_widget_destroy(dockapp);
	dockapp = (GtkWidget *) build_dockapp(data->configfile);
}


// the function to parse the config file for parameters *********************************************
int parse_from_config
		(char *filename, 
		char *param, 
		char *value)
{
	int c = 0;
	FILE *file;
	char 	sep[16], 
			buffer[128], 
			line[1024], 
			val[128];
	
	if((file=fopen(filename, "rb")) == 0) {
		fprintf (stderr,"Error opening file %s !\n",filename);
		exit(EXIT_FAILURE);
	}
	
	while (fgets (line, sizeof(line), file) !=NULL) {
		if (strstr (line, "[") !=0) continue;
		if (strstr (line, "#") !=0) continue;
		memset(val, 0, sizeof(val));
		c = sscanf (line, "%s %s %s", buffer, sep, val); 

		if (strncmp (param, buffer, sizeof(buffer)) == 0) {
			memset(value, 0, sizeof(value));

			strncpy(value,line + strlen(buffer) + strlen(sep) +2, 128);
			//fprintf(stderr,"%s\n", value);
			char *p = strchr (value, '\n');
   			if (p){
				*p = 0;
			}
			fclose (file);
			return(EXIT_SUCCESS);
		}
	memset(line, 0, sizeof(line));	
	}
	fclose (file);
	return(EXIT_FAILURE);
} // end parse_from_config


// the function to parse the config file to replace the value of a given parameter ****************************
int replace_value_in_config
		(char *filename, 
		char *param, 
		char *new_value)
{
	int c = 0;
	FILE *file;
	char 	bigbuffer[4096], 
		sep[16], 
		buffer[128], 
		line[1024], 
		old_value[128];
	
	if((file=fopen(filename, "rb")) == 0) {
		fprintf (stderr,"Error opening file %s !\n",filename);
		exit(EXIT_FAILURE);
	}
	
	memset(bigbuffer, 0, sizeof(bigbuffer));
	while (fgets (line, sizeof(line), file) !=NULL) {
		memset(buffer, 0, sizeof(buffer));
		c = sscanf (line, "%s %s %s\n", buffer, sep, old_value); 

		if (strncmp (param, buffer, sizeof(buffer)) == 0) {
			memset(line, 0, sizeof(line));
			sprintf(line, "%s %s %s\n", buffer, sep, new_value);

		}
		strncat (bigbuffer, line, sizeof(line)); 
		memset(line, 0, sizeof(line));
	}
	fclose (file);

	if((file=fopen(filename, "w+")) == 0) {
		fprintf (stderr,"Error opening file %s !\n",filename);
		exit(EXIT_FAILURE);
	}
	fprintf(file,bigbuffer);
	fclose (file);
	memset(bigbuffer, 0, sizeof(bigbuffer));
	
return(EXIT_SUCCESS);
} // end replace_value_in_config *************************************************************************






int usage(char *homecf, char *defaultcf) {
	printf("\nminidock v1.9 : 4 buttons launcher in a single dockapp\n");
	printf("Usage : \n");
	printf("- single LEFT click on a button : launch command\n");
	printf("- double RIGHT click on a button : change command & icon\n");
	printf("---------------\n");
	printf("How to start it : \n"
			"\t minidock [-c path_to_configfile] use a custom config\n"
			"\t minidock [-h] display this help\n");
	printf("---------------\n");
	printf("home config file is %s\n", homecf);
	printf("default config file is %s\n",defaultcf);
	return(EXIT_SUCCESS);
} // end usage
//*************************************************************************



// this one just copy sfile into dfile :)
int fcopy(	char *sfile, 
	char *dfile){
int c;			/* Character read/written between files */
FILE *IPFile;			/* Pointer to the I/P file. FILE is a 
				   structure  defined in <stdio.h>	*/
FILE *OPFile;

				/* Open the file - */
if ((IPFile = fopen(sfile,"r")) == 0) {
	fprintf (stderr,"Error opening file %s !\n",sfile);
	return (EXIT_FAILURE);
}
if ((OPFile = fopen(dfile,"w")) == 0) {
	fprintf (stderr,"Error opening file %s !\n",dfile);
	return (EXIT_FAILURE);
}
				/* Read one character at a time, checking 
				   for the End of File. EOF is defined 
				   in <stdio.h>  as -1 			*/
while ((c = fgetc(IPFile)) != EOF){
   fputc(c, OPFile);		/* O/P the character 			*/
   }

fclose(IPFile);		/* Close the files.			*/
fclose(OPFile);		/* Close the files.			*/
return (EXIT_SUCCESS);
} // end fcopy

